#![cfg(target_arch = "x86_64")]
#![allow(dead_code)]

use x86_64::instructions::port::Port;
use spin::Mutex;

pub const COM1: u16 = 0x3f8;

static SERIAL_DATA: Mutex<Port<u8>> = Mutex::new(Port::new(COM1));

use core::arch::asm;

pub unsafe fn io_delay() -> () {
    let mut delay_port: Port<u8> = Port::new(0x80);
    delay_port.write(0);
}

pub mod serial {
    use super::*;

    unsafe fn wait() -> () {
        let mut port: Port<u8> = Port::new(COM1 + 5);
        while (port.read() & 0x20) == 0 {
            asm!("pause");
        }
    }
    
    pub fn putchar(c: u8) -> () {
        unsafe { wait(); }
        unsafe { SERIAL_DATA.lock().write(c); }
    }
}